/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorization

import (
	"encoding/json"
	"errors"
	"net/url"

	"gitee.com/yansen_zh/wxcomponent/config"
	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// AuthorizerInfoResult 获取授权帐号信息结果
type AuthorizerInfoResult struct {
	wxerr.Error
	AuthorizationInfo AuthorizationInfo `json:"authorization_info"`
	AuthorizerInfo    AuthorizerInfo    `json:"authorizer_info"`
}

const (
	apiGetAuthorizerInfo = "https://apies.weixin.qq.com/cgi-bin/component/api_get_authorizer_info"
)

// GetAuthorizerInfo 获取授权帐号信息
func GetAuthorizerInfo(authorizerAppID string) (*AuthorizerInfoResult, error) {
	if authorizerAppID == "" {
		return nil, errors.New("获取授权帐号信息 授权方 appid 不能为空")
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", config.GetAccessToken())

	data := map[string]string{
		"component_appid":  config.GetAppID(),
		"authorizer_appid": authorizerAppID,
	}

	resp, e2 := request.PostJSON(apiGetAuthorizerInfo, &queryParam, data)
	if e2 != nil {
		return nil, e2
	}

	result := AuthorizerInfoResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	return &result, nil
}
