/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorization

import (
	"encoding/json"
	"net/url"

	"gitee.com/yansen_zh/wxcomponent/config"
	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// PreAuthCodeResult 获取预授权码结果
type PreAuthCodeResult struct {
	wxerr.Error
	PreAuthCode string `json:"pre_auth_code"`
	ExpiresIn   int    `json:"expires_in"`
}

const (
	apiCreatePreauthcode = "https://apies.weixin.qq.com/cgi-bin/component/api_create_preauthcode"
)

// CreatePreAuthCode 获取预授权码
func CreatePreAuthCode() (*PreAuthCodeResult, error) {
	queryParam := url.Values{}
	queryParam.Set("component_access_token", config.GetAccessToken())

	data := map[string]string{
		"component_appid": config.GetAppID(),
	}

	resp, e2 := request.PostJSON(apiCreatePreauthcode, &queryParam, data)
	if e2 != nil {
		return nil, e2
	}

	result := PreAuthCodeResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	return &result, nil
}
