/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorizer

import (
	"encoding/json"
	"net/url"

	"gitee.com/yansen_zh/wxcomponent/config"
	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// AuthorizerListResult 拉取所有已授权的帐号信息结果
type AuthorizerListResult struct {
	wxerr.Error
	TotalCount int                   `json:"total_count"`
	List       []AuthorizerBasicInfo `json:"list"`
}

const (
	apiGetAuthorizerList = "https://apies.weixin.qq.com/cgi-bin/component/api_get_authorizer_list"
)

// GetAuthorizerList 拉取所有已授权的帐号信息
func GetAuthorizerList(offset, count int) (*AuthorizerListResult, error) {
	if offset < 0 {
		offset = 0
	}

	if count < 0 {
		count = 100
	}

	if count > 500 {
		count = 500
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", config.GetAccessToken())

	data := map[string]interface{}{
		"component_appid": config.GetAppID(),
		"offset":          offset,
		"count":           count,
	}

	resp, e2 := request.PostJSON(apiGetAuthorizerList, &queryParam, data)
	if e2 != nil {
		return nil, e2
	}

	result := AuthorizerListResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	return &result, nil
}
